# Copyright (c) 2011 SEOmoz
# Copyright (c) 2016 leovp
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

cdef extern from "bloom.h":
    ctypedef unsigned int uint32_t
    ctypedef unsigned long int uint64_t

    ctypedef struct redisContext:
        int err
        char errstr[128]
        int fd
        int flags
        char *obuf

    ctypedef struct pyrebloomctxt:
        uint32_t        capacity
        uint32_t        hashes
        uint32_t        num_keys
        uint64_t        bits
        double          error
        uint32_t      * seeds
        char          * key
        char          * password
        redisContext  * ctxt
        char         ** keys

    bint init_pyrebloom(pyrebloomctxt *ctxt, unsigned char *key,
        uint32_t capacity, float error, char *host, uint32_t port,
        char *password, uint32_t db)
    bint free_pyrebloom(pyrebloomctxt *ctxt)

    bint add(pyrebloomctxt *ctxt, char *data, uint32_t data_size)
    bint add_one(pyrebloomctxt *ctxt, char *data, uint32_t data_size)
    int add_complete(pyrebloomctxt *ctxt, uint32_t count)

    bint check(pyrebloomctxt *ctxt, char *data, uint32_t data_size)
    int check_next(pyrebloomctxt *ctxt)

    bint delete(pyrebloomctxt *ctxt)

    uint64_t hash(unsigned char *data, uint32_t data_size, uint64_t seed, uint64_t bits)



# Copyright (c) 2011 SEOmoz
# Copyright (c) 2016 leovp
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import math
import random

cimport bloom


class PyreBloomException(Exception):
    """Some sort of exception has happened internally"""
    pass


cdef class PyreBloom(object):
    cdef bloom.pyrebloomctxt context
    cdef bytes               key

    property bits:
        def __get__(self):
            return self.context.bits

    property hashes:
        def __get__(self):
            return self.context.hashes

    def __cinit__(self, key, capacity, error, host=b'127.0.0.1', port=6379,
                  password=b'', db=0):
        if not isinstance(key, bytes):
            raise TypeError('Only byte-strings are allowed!')

        self.key = key
        if bloom.init_pyrebloom(&self.context, self.key, capacity,
                                error, host, port, password, db):
            raise PyreBloomException(self.context.ctxt.errstr)

    def __dealloc__(self):
        bloom.free_pyrebloom(&self.context)

    def delete(self):
        bloom.delete(&self.context)

    def add(self, value):
        if not isinstance(value, bytes):
            raise TypeError('Only byte-strings are allowed!')

        r = bloom.add_one(&self.context, value, len(value))
        if r < 0:
            raise PyreBloomException(self.context.ctxt.errstr)

        return r

    def update(self, values):
        r = [bloom.add(&self.context, value, len(value)) for value in values]
        r = bloom.add_complete(&self.context, len(values))
        if r < 0:
            raise PyreBloomException(self.context.ctxt.errstr)

        return r

    def intersection(self, values):
        r = [bloom.check(&self.context, value, len(value)) for value in values]
        r = [bloom.check_next(&self.context) for _ in range(len(values))]
        if min(r) < 0:
            raise PyreBloomException(self.context.ctxt.errstr)

        return [v for v, included in zip(values, r) if included]

    def contains(self, value):
        bloom.check(&self.context, value, len(value))
        r = bloom.check_next(&self.context)
        if r < 0:
            raise PyreBloomException(self.context.ctxt.errstr)

        return bool(r)

    def __contains__(self, value):
        return self.contains(value)

    def __and__(self, values):
        return self.intersection(values)

    def __iadd__(self, values):
        self.update(values)
        return self

    def keys(self):
        """Return a list of the keys used in this bloom filter"""
        return [self.context.keys[i] for i in range(self.context.num_keys)]



import setuptools
try: # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements
from pip.download import PipSession

ext_files = ['pyreBloom/bloom.c']

kwargs = {}

try:
    from Cython.Distutils import build_ext
    from Cython.Distutils import Extension

    print('Building from Cython')
    ext_files.append('pyreBloom/pyreBloom.pyx')
    kwargs['cmdclass'] = {'build_ext': build_ext}
except ImportError:
    from distutils.core import Extension

    ext_files.append('pyreBloom/pyreBloom.c')
    print('Building from C')

ext_modules = [Extension("pyreBloom", ext_files, libraries=['hiredis'],
                         library_dirs=['/usr/local/lib'],
                         include_dirs=['/usr/local/include'],
                         extra_compile_args=['-std=c99'])]

setuptools.setup(
    name='pyreBloom-ng',
    version='0.0.1',
    url='https://github.com/leovp/pyreBloom-ng',
    license='MIT',

    author='Leonid Runyshkin',
    author_email='runyshkin@gmail.com',

    keywords='bloom filter redis',
    description='Python library which implements a Redis-backed Bloom filter.',
    # long_description=read('README.rst'),

    platforms=['any'],
    ext_modules=ext_modules,

    install_requires=[
        str(req.req) for req in parse_requirements('requirements.txt',
                                                   session=PipSession())
        ],

    tests_require=[
        str(req.req) for req in parse_requirements('requirements_test.txt',
                                                   session=PipSession())
        ],

    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
        'Programming Language :: C',
        'Programming Language :: Cython',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)
